package cscie160.project;

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.UnknownHostException;
import java.rmi.server.UnicastRemoteObject;

/** cscie160 libraries */
import cscie160.project.atm.ATMFactory;
import cscie160.project.atm.ATM;
import cscie160.project.account.AccountInfo;
import cscie160.project.atm.ATMListener;

/**
  * By David Cheung <p>
  * CSCIE160 Project <p>
  * DUE 05-16-2001 <p>
  * <p>
  * Client class implements ATMListener so that
  * it will do callbacks as required from the project
  * spec. <p>
  * This also performs the registration to the ATM. <p>
  * <p>
  * in Main: <p>
  * Client first contact ATMFactory instance. <p>
  * Second, register itself to the ATM. The ATM provides
  * methods to store the remote object in a vector. <p>
  * Finally, perform the tests required in project spec. <p>
  */

public class Client 
	extends UnicastRemoteObject 
	implements ATMListener 
{
	/** 
	  * constructor: <p>
	  * need to call super() for RMI objects.
	  */
	public Client()
		throws RemoteException
	{	
		super();
	}
	
	/** 
	  * Then this function is called, it will print out
	  * messages from ATM.
	  */
	public void getNotify(TransactionNotification noteBox)
		throws java.rmi.RemoteException
	{
		System.out.println(noteBox);
	}
	/**
	  * registers itself to the list of callbacks.
	  * calls addCallBack function from ATM.
	  */
	public void registerMe(ATM atmA)
	{
		try
		{
			atmA.addCallBack((ATMListener)this);
		}
		catch(RemoteException exc)
		{
			System.out.println("Error in lookup() " + exc.toString());
		}
	}
	/**
	  * creates an AccountInfo based on acctnum and pin and then
	  * returns it.
	  */
	public static AccountInfo getAccountInfo(int acctNum, int acctPin)
	{
		return new AccountInfo(acctNum,acctPin);
	}	
   /**
     * main:  <p>
     * 1) call ATM class. <p>
     * 2) lookup ATM factory. <p>
     * 3) use getATM from ATMfactory and stores it. <p>
     * 4) Register itself to ATM. <p>
     * 5) Perform tests. <p.
     * <p>
     *  Modified some negative tests by adding "should fail!" messages
     * when test is suppose to fail. this is useful when test passed
     * when it should fail. <p>
     * <p>
     */
	public static void main(String[] args)
	{
		ATM atm = null;

		try
		{
			Client clientA = new Client();
			
			ATMFactory factory = (ATMFactory)Naming.lookup("//localhost/atmfactory");
			atm = factory.getATM();
			
			//register me to the server!
			clientA.registerMe(atm);
		} catch(MalformedURLException mue)
		{
			mue.printStackTrace();
		} catch(NotBoundException nbe)
		{
			nbe.printStackTrace();
		} catch(UnknownHostException uhe)
		{
			uhe.printStackTrace();
		} catch(RemoteException re)
		{
			re.printStackTrace();
		}
		if(atm!=null)
		{
			testATM(atm);
		}
		System.out.println("Done! Exiting....?");
	}

   public static void testATM(ATM atm) {
      if (atm!=null) {
         printBalances(atm);
         performTestOne(atm);
         performTestTwo(atm);
         performTestThree(atm);
         performTestFour(atm);
         performTestFive(atm);
         performTestSix(atm);
         performTestSeven(atm);
         performTestEight(atm);
         performTestNine(atm);
         printBalances(atm);
      }
   }        
   public static void printBalances(ATM atm) {        
      try {
         System.out.println("Balance(0000001): "+atm.getBalance(getAccountInfo(0000001, 1234)));
         System.out.println("Balance(0000002): "+atm.getBalance(getAccountInfo(0000002, 2345)));
         System.out.println("Balance(0000003): "+atm.getBalance(getAccountInfo(0000003, 3456)));
      } catch (Exception e) {
         e.printStackTrace();
      }
   }
   public static void performTestOne(ATM atm) {       
      try {
         System.out.println("Test 1");
         atm.getBalance(getAccountInfo(0000001, 5555));
         System.out.println("This should fail!");
      } catch (Exception e) {
         System.out.println("Failed as expected: "+e);
      }
   }
   public static void performTestTwo(ATM atm) {       
      try {
         System.out.println("Test 2");
         atm.withdraw(getAccountInfo(0000002, 2345), 500);
         System.out.println("This should fail!");

      } catch (Exception e) {
         System.out.println("Failed as expected: "+e);
      }
   }
   public static void performTestThree(ATM atm) {        
      try {
         System.out.println("Test 3");
         atm.withdraw(getAccountInfo(0000001, 1234), 50);
         
         System.out.println("This should fail!"); 
         
      } catch (Exception e) {
         System.out.println("Failed as expected: "+e);
      }
   }
   public static void performTestFour(ATM atm) {         
      try {
         System.out.println("Test 4");
         atm.deposit(getAccountInfo(0000001, 1234), 500);
      } catch (Exception e) {
         System.out.println("Unexpected error: "+e);
      }
   }
   public static void performTestFive(ATM atm) {         
      try {
         System.out.println("Test 5");
         atm.deposit(getAccountInfo(0000002, 2345), 100);
      } catch (Exception e) {
         System.out.println("Unexpected error: "+e);
      }
   }
   public static void performTestSix(ATM atm) {       
      try {
         System.out.println("Test 6");
         atm.withdraw(getAccountInfo(0000001, 1234), 100);
      } catch (Exception e) {
         System.out.println("Unexpected error: "+e);
      }
   }
   public static void performTestSeven(ATM atm) {        
      try {
         System.out.println("Test 7");
         atm.withdraw(getAccountInfo(0000003, 3456), 300);
      } catch (Exception e) {
         System.out.println("Unexpected error: "+e);
      }
   }
   public static void performTestEight(ATM atm) {        
      try {
         System.out.println("Test 8");
         atm.withdraw(getAccountInfo(0000001, 1234), 200);
         System.out.println("This should fail!");

      } catch (Exception e) {
         System.out.println("Failed as expected: "+e);
      }
   }
   public static void performTestNine(ATM atm) {        
      try {
         System.out.println("Test 9");
         atm.transfer(getAccountInfo(0000001, 1234),getAccountInfo(0000002, 2345), 100);
      } catch (Exception e) {
         System.out.println("Unexpected error: "+e);
      }
   }
}

