package cscie160.project.bank;

import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.util.Hashtable;

/** cscie libraries */
import cscie160.project.account.Account;
import cscie160.project.account.AccountImpl;

/**
  * By David Cheung <p>
  * CSCIE160 Project <p>
  * 05-16-2001 <p>
  *
  * BankImpl class: <p>
  * Implementes Bank interface.  <p>
  * This class is important for Account store house. <p>
  * Used to create and retrieve accounts. <p>
  * The functions are called by ATM. <p>
  */
public class BankImpl 
		extends UnicastRemoteObject
		implements Bank
{
	/** Database to store acounts */
	private Hashtable accountDB;
	
	/** the contructor does the following: <p>
	  * call super() <p>
	  * initialize hashtable <p>
	  * creates the 3 accounts required in project spec. <p>
	  */	
	public BankImpl() 
		throws RemoteException
	{
		super();
		accountDB = new Hashtable();
		createAccount(000001,0);
		createAccount(000002,100);
		createAccount(000003,500);		
	}
	/** 
	  *	this does not do security check ie, require
	  * AccountInfo object because that is already
	  * checked by the security object.
	  * So, it is the ATM's responsibility to check the validity
	  * of an account.
	  * 
	  */
	public Account getAccount(int acctNum)
		throws java.rmi.RemoteException
	{
		try
		{
			return (Account)accountDB.get(new Integer(acctNum));
		}
		catch (java.lang.NullPointerException npe)
		{
			System.out.println("The Account doesn't exist!");
			return null;
		}
	}
	
	/** Creates account based on accountNumber and initial balance */
	public void createAccount(int accountNum, int initBalance)
	{
		try
		{
			AccountImpl newAccount;
			newAccount = new AccountImpl(accountNum, initBalance);
			accountDB.put(new Integer(accountNum), newAccount);
		}
		catch(java.rmi.RemoteException re)
		{
			System.out.println("Remote Exception! \nError creating account.");
			re.printStackTrace();
		}
	}
	
	
	//test bed
	/*
	public static void main (String argv[])
	{
		try
		{
		BankImpl bankA= new BankImpl();
		bankA.createAccount(000001, 0);
		bankA.createAccount(000002, 50);
		bankA.createAccount(00003, 500);
		AccountImpl acctA = bankA.getAccount(000001);
		System.out.println(" account 1:" + acctA.getBalance());
		System.out.println(" account 2:" + (bankA.getAccount(000002)).getBalance());
		System.out.println(" account 4:" + (bankA.getAccount(000004)).getBalance());
		System.out.println(" account 3:" + (bankA.getAccount(000003)).getBalance());
		
		}
		catch (RemoteException re)
		{
			re.printStackTrace();
		}
		catch (java.lang.NullPointerException npe)
		{
			System.out.println("Account don't exist!");
		}
	
	}
	*/
}
		