package cscie160.project.security;

import java.util.Hashtable;

/** cscie libraries */
import cscie160.project.account.AccountInfo;
import cscie160.project.account.AccountImpl;
import cscie160.project.security.Security;
import cscie160.project.security.exception.SecurityInvalidPinException;

/**
  * By David Cheung <p>
  * CSCIE160 Project <p>
  * DUE 05-16-2001 <p>
  * <p>
  * This class Implements the Security class. This is suppose to reside
  * on the server side.  It contains a private class "authorizeClass"
  * to encapsulate the information for different types of permissions. <p>
  * Also contains a private hashtable to store account permissions and such.
  */
public class SecurityImpl 
		extends java.rmi.server.UnicastRemoteObject
		implements Security
{
	/**
	  * authorizeClass stores the pin and permissions
	  * in a single class. The objects in there is public
	  * but the class itself is private. This way other methods
	  * cannot use it.
	  */
	private class authorizeClass
	{
		public boolean depositOK;
		public boolean withdrawOK;
		public boolean balanceOK;
		public int acctPin;
	}
	/** Hashtable to store accounts. <p>
	  * I used the account number as the key instead
	  * of the accountinfo class because it is diffferent
	  * even if pin and accountnumber is the same.
	  */
	
	private Hashtable DBtable;
	
	/** Constructor: <p>
	  * 1) calls super <p>
	  * 2) initialize database <p>
	  * 3) create 3 accounts base on spec. <p>
	  */
	public SecurityImpl()
			 throws java.rmi.RemoteException
	{
		super();
		DBtable = new Hashtable();
		//create initial accounts
		this.createSec(new AccountInfo(0000001,1234),true, true, true);
		this.createSec(new AccountInfo(0000002,2345),true, false, true);
		this.createSec(new AccountInfo(0000003,3456),false, true, true);
	}
	
	  
	/**
	  * this function creates an authorize class and stores it into the 
	  * hash table. Uses the accountNumber as the key of the hashtable.
	  */
	public void createSec(AccountInfo acct, boolean depPermission, 
			boolean withPermission, boolean balPermission)
			throws java.rmi.RemoteException
	{
		authorizeClass tmpAuthorize;
		tmpAuthorize = new authorizeClass();
		tmpAuthorize.depositOK = depPermission;
		tmpAuthorize.withdrawOK = withPermission;
		tmpAuthorize.balanceOK = balPermission;
		tmpAuthorize.acctPin=acct.getPinNum();
		
		DBtable.put(new Integer(acct.getAccountNum()), tmpAuthorize);
	
	}
	
	/** 
	  * This function returns a boolean to determine if deposit
	  * permission of an AccountInfo is permissible. 
	  */
	public boolean getDepPermission(AccountInfo acct)
		throws java.rmi.RemoteException
	{
		try
		{
			authorizeClass tmpAuthA;
      	tmpAuthA = (authorizeClass)DBtable.get(new Integer(acct.getAccountNum()));
     		return tmpAuthA.depositOK;
	   }
	   catch(java.lang.NullPointerException npe)
	   {
	   	System.out.println("Error! Incorrect pin!");
		   return false;
	   }

	}
	/** 
	  * This function returns a boolean to determine if withdraw
	  * permission of an AccountInfo is permissible. 
	  */

	public boolean getWithPermission(AccountInfo acct)
		throws java.rmi.RemoteException

	{
		try
		{
			authorizeClass tmpAuthA;
      	tmpAuthA = (authorizeClass)DBtable.get(new Integer(acct.getAccountNum()));
     		return tmpAuthA.withdrawOK;
	   }
	   catch(java.lang.NullPointerException npe)
	   {
	   	System.out.println("Error! Incorrect pin!");
		   return false;
	   }

	}
	/** 
	  * This function returns a boolean to determine if getBalance
	  * permission of an AccountInfo is permissible. 
	  */

	public boolean getBalPermission(AccountInfo acct)
		throws java.rmi.RemoteException
	{
		try
		{
			authorizeClass tmpAuthA;
      	tmpAuthA = (authorizeClass)DBtable.get(new Integer(acct.getAccountNum()));
     		return tmpAuthA.balanceOK;
	   }
	   catch(java.lang.NullPointerException npe)
	   {
	   	System.out.println("Error! Incorrect pin!");
		   return false;
	   }

	}
	
	/**
	  * Returns true if an account with the accountInfo
	  * exists, otherwise return false.
	  * <p>
	  * This is use to verify if the pin and the accountnumber
	  * match.
	  */
	public boolean verifyAccount(AccountInfo acct)
		throws java.rmi.RemoteException
	{
		try
		{
			authorizeClass tmpAuthA;
			tmpAuthA = (authorizeClass)DBtable.get(new Integer(acct.getAccountNum()));
			if (tmpAuthA.acctPin == acct.getPinNum()) return true;
			else return false; /** this part never really get executed... */
		}
		catch(java.lang.NullPointerException npe)
		{
			/** pin is not correct */
			return false;
		}
	}


	// Test bed
	/*
	public static void main(String argv[])
	{
		//create a security class
		try
		{
			SecurityImpl secA = new SecurityImpl();
			AccountInfo acctA = new AccountInfo(0000001,1234);
			AccountInfo acctB = new AccountInfo(0000001,2345);
			AccountInfo acctC = new AccountInfo(0000002,3456);
		
		//	secA.createSec(acctA, true, true, true);
			
			if (secA.verifyAccount(acctA))
			{
				System.out.println("Account verified OK!");
			}else System.out.println("Account verified Not OK!");

			if (secA.verifyAccount(acctB))
			{
				System.out.println("Account verified OK!");
			}else System.out.println("Account verified Not OK!");

			if (secA.getBalPermission(acctA))
			{
				System.out.println("Balance Permission OK!");
			}else System.out.println("Balance Permission Not OK!");

			if (secA.getBalPermission(acctC))
			{
				System.out.println("Balance Permission OK!");
			}else System.out.println("Balance Permission Not OK!");


		}
		catch(java.rmi.RemoteException re)
		{
			System.out.println("remote Exception!");
			re.printStackTrace();
		}
		
	}
	*/
}